import java.util.Arrays;
import java.util.HashMap;
import java.util.LinkedList;
import java.util.Queue;
import java.util.Scanner;
import java.util.Stack;
import java.util.StringTokenizer;

public class Logic_DA {

	class Op {
		String name;
		boolean isUnary;
		int[][] tt;

		Op(String name, boolean isUnary, int[][] tt) {
			this.name = name;
			this.isUnary = isUnary;
			this.tt = tt;
			if (isUnary) {
				if (tt[0][1] == tt[0][2])
					tt[0][0] = tt[0][0];
			} else {
				if (tt[1][1] == tt[1][2])
					tt[1][0] = tt[1][1];
				if (tt[2][1] == tt[2][2])
					tt[2][0] = tt[2][2];
				if (tt[1][1] == tt[2][1])
					tt[0][1] = tt[1][1];
				if (tt[1][2] == tt[2][2])
					tt[0][2] = tt[2][2];
				if (tt[1][1] == tt[1][2] && tt[1][2] == tt[2][2]
						&& tt[2][2] == tt[2][1])
					tt[0][0] = tt[1][1];
			}
		}

		public int getResult(int... operands) {
			if (isUnary)
				return tt[0][operands[0]];
			return tt[operands[0]][operands[1]];
		}

		public String toString() {
			return name;
		}
	}

	private Scanner sc = new Scanner(System.in);

	private HashMap<String, Op> ops = new HashMap<String, Op>();
	private int[] val = new int[26];

	private Queue<Object> q = new LinkedList<Object>();

	private static final int UNKNOWN = 0;
	private static final int FALSE = 1;
	private static final int TRUE = 2;
	private static final String[] ANS = { "unknown", "false", "true" };

	private void getOps(int u, int b) {
		ops.clear();
		while (u-- > 0) {
			String name = sc.nextLine().trim();
			int[][] tt = new int[1][3];
			StringTokenizer st = new StringTokenizer(sc.nextLine().trim());
			if (st.nextToken().charAt(0) == 'f')
				tt[0][1] = FALSE;
			else
				tt[0][1] = TRUE;
			if (st.nextToken().charAt(0) == 'f')
				tt[0][2] = FALSE;
			else
				tt[0][2] = TRUE;
			ops.put(name, new Op(name, true, tt));
		}

		while (b-- > 0) {
			String name = sc.nextLine().trim();
			int[][] tt = new int[3][3];
			for (int i = 1; i <= 2; i++) {
				StringTokenizer st = new StringTokenizer(sc.nextLine().trim());
				for (int j = 1; j <= 2; j++) {
					if (st.nextToken().charAt(0) == 'f')
						tt[i][j] = FALSE;
					else
						tt[i][j] = TRUE;
				}
			}
			ops.put(name, new Op(name, false, tt));
		}
	}

	private void getValues() {
		Arrays.fill(val, UNKNOWN);
		while (true) {
			StringTokenizer st = new StringTokenizer(sc.nextLine().trim());
			if (st.countTokens() != 2)
				break;
			int var = st.nextToken().charAt(0) - 'a';
			val[var] = st.nextToken().charAt(0) == 'f' ? FALSE : TRUE;
		}
	}

	private void convertToPrefix(char[] expr) {
		q.clear();
		Stack<Object> s = new Stack<Object>();
		char[] curOp = new char[512];
		int curOpLen = 0;
		for (int i = 0; i < expr.length; i++) {
			if (expr[i] != ' ') {
				if ('A' <= expr[i] && expr[i] <= 'Z') {
					curOp[curOpLen++] = expr[i];
				} else {
					if (curOpLen > 0) {
						while (!s.isEmpty() && s.peek() instanceof Op) {
							q.add(s.pop());
						}
						s.push(ops.get(new String(curOp, 0, curOpLen)));
						curOpLen = 0;
					}
					switch (expr[i]) {
					case '(':
						s.push("(");
						break;
					case ')':
						while (!s.isEmpty() && s.peek() instanceof Op) {
							q.add(s.pop());
						}
						s.pop();
						break;
					default:
						q.add(Integer.valueOf(val[expr[i] - 'a']));
						break;
					}
				}
			}
		}
		while (!s.isEmpty()) {
			q.add(s.pop());
		}
	}

	private int eval() {

		int[] s = new int[512];
		int sz = 0;
		int[] operands = new int[2];
		for (Object o : q) {
			if (o instanceof Op) {
				Op op = (Op) o;
				if (op.isUnary) {
					operands[0] = s[--sz];
				} else {
					operands[1] = s[--sz];
					operands[0] = s[--sz];
				}
				s[sz++] = op.getResult(operands);
			} else {
				s[sz++] = ((Integer) o).intValue();
			}
		}
		return s[0];
	}

	private void work() {
		int tc = 1;
		while (true) {
			StringTokenizer st = new StringTokenizer(sc.nextLine().trim());
			int u = Integer.parseInt(st.nextToken());
			if (u < 0)
				break;
			int b = Integer.parseInt(st.nextToken());
			getOps(u, b);
			char[] expr = sc.nextLine().trim().toCharArray();
			getValues();
			convertToPrefix(expr);
			System.out.printf("Case %d: %s\n", tc++, ANS[eval()]);
		}
		sc.close();
		System.out.close();
	}

	public static void main(String[] args) {
		new Logic_DA().work();
	}

}
