import java.util.Arrays;
import java.util.LinkedList;
import java.util.Queue;
import java.util.Scanner;

public class Maze_DA {
	class State {
		int r, c, t, mask;
		State prev;

		public State(int r, int c, int t, int mask, State prev) {
			this.r = r;
			this.c = c;
			this.t = t;
			this.mask = mask;
			this.prev = prev;
		}

		public int hashCode() {
			final int prime = 31;
			int result = 1;
			result = prime * result + c;
			result = prime * result + mask;
			result = prime * result + r;
			result = prime * result + t;
			return result;
		}

		public boolean equals(Object obj) {
			State other = (State) obj;
			if (c != other.c)
				return false;
			if (mask != other.mask)
				return false;
			if (r != other.r)
				return false;
			if (t != other.t)
				return false;
			return true;
		}

		public String toString() {
			return String.format("(%d,%d) %s open:%s d=%d", r, c,
					Integer.toBinaryString(mask),
					nesw.charAt((t + grid[r][c]) & 3), d[t][r][c][mask]);
		}

	}

	private static final int MAX_RC = 100;
	private static final int oo = Integer.MAX_VALUE / 4;

	String nesw = "NESW";
	int[] dr = { -1, 0, 1, 0 };
	int[] dc = { 0, 1, 0, -1 };

	Queue<State> q = new LinkedList<State>();
	int[][] grid = new int[MAX_RC][MAX_RC];
	int[][] treasures = new int[MAX_RC][MAX_RC];
	int[][][][] d = new int[4][MAX_RC][MAX_RC][1 << 8];

	private void work() {
		Scanner sc = new Scanner(System.in);

		while (true) {
			int m = sc.nextInt();
			int n = sc.nextInt();
			if (m == 0 && n == 0)
				break;

			for (int i = 0; i < m; i++) {
				char[] line = sc.next().toCharArray();
				for (int j = 0; j < n; j++) {
					grid[i][j] = nesw.indexOf(line[j]);
					treasures[i][j] = -1;
					for (int k = 0; k < 4; k++) {
						Arrays.fill(d[k][i][j], oo);
					}
				}
			}

			int k = sc.nextInt();
			for (int i = 0; i < k; i++) {
				int r = sc.nextInt() - 1;
				int c = sc.nextInt() - 1;
				treasures[r][c] = i;
			}
			int goalMask = (1 << k) - 1;

			q.clear();
			State cur = new State(0, 0, 0, 0, null);
			q.add(cur);
			d[0][0][0][0] = 0;
			int res = -1;
			while (!q.isEmpty()) {
				cur = q.poll();
				if (cur.r == m - 1 && cur.c == n - 1 && cur.mask == goalMask) {
					res = d[cur.t][m - 1][n - 1][goalMask];
					break;
				}

				int nt = (cur.t + 1) & 3;

				// wait here
				if (d[nt][cur.r][cur.c][cur.mask] > d[cur.t][cur.r][cur.c][cur.mask] + 1) {
					q.add(new State(cur.r, cur.c, nt, cur.mask, cur));
					d[nt][cur.r][cur.c][cur.mask] = d[cur.t][cur.r][cur.c][cur.mask] + 1;
				}

				// try to move
				int dir = (cur.t + grid[cur.r][cur.c]) & 3;
				int nr = cur.r + dr[dir];
				int nc = cur.c + dc[dir];
				if (0 <= nr && nr < m && 0 <= nc && nc < n) {
					int nmask = cur.mask;
					if (treasures[nr][nc] >= 0) {
						nmask |= 1 << treasures[nr][nc];
					}
					if (d[nt][nr][nc][nmask] > d[cur.t][cur.r][cur.c][cur.mask] + 1) {
						q.add(new State(nr, nc, nt, nmask, cur));
						d[nt][nr][nc][nmask] = d[cur.t][cur.r][cur.c][cur.mask] + 1;
					}
				}
			}
			System.out.println(res);
			// print(cur);
		}
	}

	private void print(State s) {
		if (s == null)
			return;
		print(s.prev);
		System.out.println(s);
	}

	public static void main(String[] args) {
		long startTime = System.currentTimeMillis();
		new Maze_DA().work();
		// System.out.printf("time: %.2f\n",
		// 0.001 * (System.currentTimeMillis() - startTime));
	}

}
